# CellTracking alignment module
# This module is a standalone command line module
# By MW Aug. 2012 GPLv3+

print "USAGE : python CT_views.py ./mosaic_instruction_file"
print "The views folder contains a example.js file showing how the alignemnent\n\
instruction file should be stuctured.\n\n"

basefolder = './exports/views/'
datafile = 'data.txt'
arch = 'debian'	# 'debian' and 'mac' are supported for the moment

import sys, os, json, pickle, shutil, re

# ===== Functions
def align(nonaligned, phaseframes) :
	frames_before = []
	frames_after = []
	for i in range(len(phaseframes)) :
		frames_before.append(phaseframes[i][0])
		frames_after.append(len(nonaligned[i])-phaseframes[i][0]-1)
		print (frames_before[i], frames_after[i])
		
	if j['_frames_before_alignment'] == -1 :
		f_before = min(frames_before)
	else :
		f_before = j['_frames_before_alignment']
	if j['_frames_after_alignment'] == -1 :
		f_after = min(frames_after)
	else :
		f_after = j['_frames_after_alignment']
	print '--- Aligning'
	print (f_before, f_after)

	na = 'na'
	cropped = []
	#for i in cropped :
	#	print len(cropped)
	for i in range(len(phaseframes)) :
		index = f_after - frames_after[i]
		#print index
		if index >= 0 :
			navector = [na]*index
			cropped.append(nonaligned[i]+navector)
		else :
			cropped.append(nonaligned[i][:index])
	nonaligned = cropped
	aligned = []
	for i in range(len(phaseframes)) :
		index = frames_before[i] - f_before
		if index >= 0 :
			aligned.append(nonaligned[i][index:])
		else :
			navector = [na]*(-1*index)
			aligned.append(navector+nonaligned[i])
	return (aligned, f_before, f_after)
def _tryint(s):
	try:
		return int(s)
	except:
		return s
def _alphanum_key(s):
	""" Turn a string into a list of string and number chunks.
		"z23a" -> ["z", 23, "a"]
	"""
	return [ _tryint(c) for c in re.split('([0-9]+)', s) ]
def sort_nicely(l):
	""" Sort the given list in the way that humans expect.
	"""
	l.sort(key=_alphanum_key)

# ====== Main program
try :
	instr_file = sys.argv[1]
except :
	print "An mosaic instruction file must be specified"
if not os.path.isfile(instr_file) :
	print "File not found"
	sys.exit(1)


# ====== Opening instructions file
print "-- Opening mosaic instructions file"
f = open(instr_file, 'r').read()
try :
	j = json.loads(f)
	ret = True
	miss_key = []
	for i in ['_path', '_output_folder', "_show_labels", '_alignment_marker', "_frames_before_alignment", "_frames_after_alignment", "_title", "_shape", "_time_lapse"] :
		if not j.has_key(i) :
			ret = False
			miss_key.append(i)
except Exception, e :
	print "The mosaic file should be a valid JSON file \n" + 'Error :' + str(e)
	sys.exit(1)
if not ret :
	print "missing fields in the mosaic file : " + str(miss_key)
	sys.exit(1)

# ======== Checking files
print "-- Checking input files"
ok = True
l = []
for p in j['_path'] :
	pp = os.path.join(p[1], datafile)
	if not os.path.isfile(pp) :
		ok = False
		l.append(pp)
if not ok :
	print "Invalid path :"
	for i in l :
		print i
	sys.exit(1)

# ====== Creating list of loaded files
datafiles = []
for p in j['_path'] :
	pp = os.path.join(p[1], datafile)
	f = open(pp, 'r')
	datafiles.append(f.read())
	f.close()

savefiles = [None]*len(datafiles)
cellindexes = [None]*len(datafiles)
for i in range(len(datafiles)) :
	f = datafiles[i]
	for l in f.split('\n') :
		(ok1, ok2) = (False, False)
		if l[0:11] == 'Save file :' :
			savefiles[i] = l[11:].strip()
			ok1 = True
		if l[0:12] == 'Cell index :' :
			cellindexes[i] = l[12:].strip()
			ok2 = True
		if ok1 and ok2 :
			break	
			
		
if None in cellindexes :
	print "The following files have no valid cell index associated or the save file line cannot\
be found"
	print "A valid cell index line begins with 'Cell index :' (case and spaces ARE important)"
	print "Missing cell indexes :"
	for i in range(len(cellindexes)) :
		if cellindexes[i] == None :
			print j['_path'][i]
	sys.exit(1)
if None in savefiles :
	print "The following files have no valid save file associated or the save file line cannot\
be found"
	print "A valid save file line begins with 'Save file :' (case and spaces ARE important)"
	print "Missing save files :"
	for i in range(len(savefiles)) :
		if savefiles[i] == None :
			print j['_path'][i]
	sys.exit(1)
ret = True
l = []
for p in savefiles :
	if not os.path.isfile(p) :
		l.append(p)
		ret = False
if not ret :
	print "The following files cannot be found, please check locations :"
	for p in l :
		print p
	sys.exit(1)
ret = True
l = []
for i in range(len(j['_path'])) :
	try :
		cellindexes[i] = int(cellindexes[i].strip())
	except :
		ret = False
		l.append(cellindexes[i], j['_path'][i])
if not ret :
	print "The following cell indexes are invalid, please correct them :"
	for p in l :
		print "cell index : " + str(p[0]) + ', in file : ' + p[1]
	sys.exit(1)
	
# ====== Extracting alignment data
print "-- Getting alignment data"
phaseindexes = [None]*len(savefiles)
phaseframes = [None]*len(savefiles)
for i in range(len(savefiles)) :
	c = cellindexes[i]
	p = savefiles[i]
	f = open(p, 'rb')
	data = pickle.load(f)
	f.close()
	list_cells = data['cells']['value']['cells']['value']
	if len(list_cells) < c :
		print "There are too few cells registered in the .ct file... Can not continue"
		sys.exit(1)
	cell = list_cells[c]
	if cell['index']['value'] != c :
		print "Registered index is not corresponding"
		sys.exit(1)
	cell = cell['cell_data']['value']
	phases = cell['phases']['value']['phases']['value']		# A list of phases
	
	ret = False
	l = []
	
	for k in range(len(phases)) :
		phase = phases[k]['phase_data']['value']
		if phase['s_name']['value'] == j['_alignment_marker'] :
			ret = True
			phaseframes[i] = phase['phase_interval']['value']
			phaseindexes[i] = k
		else :
			l.append(phase['s_name']['value'])
	if not ret :
		print "The alignment marker '" + j['_alignment_marker'] + "' has not been found in the .ct file,"
		print "available alignment markers are :"
		for i in l :
			print i
		sys.exit(1)
if None in phaseindexes :
	print "Some alignment markers are not found, exiting"
	sys.exit(1)

# ====== Checking phase frames
for i in phaseframes :
	try :
		list(i)
	except :
		print "'phase_interval' is not a list, phase interval="
		print i
		sys.exit(1)
	try :
		for k in i :
			int(k)
	except :
		print "'phase_interval' elements are not integers, phase_interval="
		print i
		
# ====== Preparing saving
print '-- Getting file list'
imagefiles = []
for p in j['_path'] :
	l = os.listdir(p[1])
	l.pop(l.index('data.txt'))
	sort_nicely(l)
	imagefiles.append(l)
	


# ====== Aligning frames
print "-- Computing alignment"

print "Aligning frames : " + str(phaseframes)
print '--- Loading frames'
nonaligned = []		# The nonaligned values
for i in range(len(j['_path'])) :
	p = j['_path'][i][1]
	values = range(len(imagefiles[i]))
	nonaligned.append(values)
	print str(len(values)) + ' frames detected for movie ' + p

print '--- Computing alignment data'
nn = []
for i in nonaligned :
	nnn = []
	m = min(i)
	for k in i :
		nnn.append(k-m)
	nn.append(nnn)

(aligned, f_before, f_after) = align(nonaligned, phaseframes)

# ====== Creating instructions list
print "-- Creating instructions list"
finallist = []	# Format : [frame_index, movie_index1, movie_index2, ..., movie_indexn]
for i in range(len(aligned[0])) :
	l = [i]
	for k in aligned :
		l.append(k[i])
	finallist.append(l)

# ====== Calculating image sizes
print "-- Calculating image sizes"
import cv
if arch == 'mac' :
	import numpy, Image
sizes_x = []
sizes_y = []
for i in range(len(imagefiles)) :
	p = os.path.join(j['_path'][i][1], imagefiles[i][0])
	if arch == 'debian' :
		im = cv.LoadImage(p)
	elif arch == 'mac' :
		im = cv.GetImage(cv.fromarray(numpy.array(Image.open(p))))           # HACKING LINE FOR MACOS - LoadImage not working with png
	else :
		print 'Unrecognized arch argument (beginning of the file)'
		sys.exit(1)
	(x,y) = cv.GetSize(im)
	sizes_x.append(x)
	sizes_y.append(y)
rect_size = (max(sizes_x), max(sizes_y))
im_size = (j['_shape'][0]*rect_size[0], j['_shape'][1]*rect_size[1])

# ====== Creating output files/folder

	
# ====== Saving
print '-- Saving'
folder = basefolder + j['_output_folder'] + '/'
print 'Save folder is : ' + folder
if os.path.isdir(folder) :
	r = raw_input('The output folder already exists, do you want to overwrite it? (y/n) [n] ')
	if r in ['y', 'Y', 'yes', 'YES', 'Yes'] :
		shutil.rmtree(folder)
	else :
		print 'Exiting'
		sys.exit(1)
		
os.mkdir(folder)
out_f = os.path.join(folder, 'mosaic/')
os.mkdir(out_f)
shutil.copy2(instr_file, folder + os.path.basename(instr_file))

# ====== Creating output image
print "press SPACE to see next frame"
timing_info = range(-1*f_before*j['_time_lapse'], (f_after+1)*j['_time_lapse'], j['_time_lapse'])
font = cv.InitFont(cv.CV_FONT_HERSHEY_SIMPLEX, 0.3, 0.3)
color = (70, 70, 70)
for instr in finallist :
	#print '-----'
	i = instr[0]
	im = cv.CreateImage((im_size[0], im_size[1]+35), 8, 3)
	cv.SetZero(im)
	for k in range(len(j['_path'])) :
		#print (k, j['_shape'][0], j['_shape'][1])
		r = k//j['_shape'][0]	# Might be c instead of r (row)
		c = k%j['_shape'][0]
		c_size = (sizes_x[k], sizes_y[k])
		rect = (int(c*rect_size[0]+0.5*(rect_size[0]-c_size[0])),\
				int(r*rect_size[1]+0.5*(rect_size[1]-c_size[1])),\
				c_size[0],\
				c_size[1])		# Empirically speaking, this line is fine and working
		cv.SetImageROI(im, rect)
		ppp = os.path.join(j['_path'][k][1], imagefiles[k][instr[k+1]])
		if arch == 'debian' :
			cell = cv.LoadImage(ppp)
		if arch == 'mac' :
			cell = cv.GetImage(cv.fromarray(numpy.array(Image.open(ppp))))           # HACKING LINE FOR MACOS - LoadImage not working with png
		#print (cv.GetSize(cell), cv.GetSize(im))
		try :
			cv.Copy(cell, im)
		except :
			print "Copy failed, make sure that you defined properly the _shape parameter"            
		if j["_show_labels"] :
			cv.PutText(im,str(j['_path'][k][0]), (5,10),font, color)
		cv.ResetImageROI(im)
	cv.PutText(im,j['_title'], (5,im_size[1]+10),font, color)
	cv.PutText(im,'t='+str(timing_info[i])+'min', (5,im_size[1]+25),font, color)
	cv.ShowImage(str(j['_title']), im)
	cv.SaveImage(os.path.join(out_f, 'export'+str(i)+'.tif'), im)
	cv.WaitKey(0)



f_al = open(folder + 'alignment.txt', 'w')

labels2 = 'Frame, '
labels = 'Aligned frame index\nIndex, Cell index, Folder path\n'
for i in range(len(j['_path'])) :
	labels += str(i) + ', ' + str(cellindexes[i]) + ', ' + j['_path'][i][1] + '\n'
	labels2 += 'index ' + str(i) + ', '
labels2 = labels2[:-2] + '\n'
labels += '\n\n'
labels += labels2
f_al.write(labels)

# Creating output string

for i in finallist :
	out = ''
	for k in i :
		out += str(k) + ', '
	out = out[:-2]
	out += '\n'
	f_al.write(out)
f_al.close()
print '--- Everything saved! Enjoy!'
			
	
