import time, os, numpy
import pygtk
pygtk.require('2.0')
import gtk, gobject
#import gtk.glade

class Cell_GTK :
	def __init__(self, parent) :
		self.parent = parent
		self.auto_toggle = False
		
	def new_frame(self, gtk) :
		"""Creates and returns a panel containing all the controls for the management of the cell object
		It also initializes all the required mouse/button bindings.
		
		:param gtk: A :class:`gtk` instance
		:type gtk: :class:`gtk`
		
		:returns: A frame object that can be attached to a window.
		:rtype: :class:`gtk.Frame`"""
		
		self.gtk = gtk
		frame = self.gtk.Frame(str(self.parent.get_label()))
		frame.show()
		table = self.gtk.Table(3,3)
		table.show()
		frame.add(table)
		
		# Labels
		l_phase = self.gtk.Label('Phase : ')
		l_phase.show()
		table.attach(l_phase, 0,1,0,1)
		
		####ph = self._get_phase()
		ph = ('not implemented', 'pink')
		self.l_phase2 = self.gtk.Button(str(self.parent.get_label()))
		color = self.gtk.gdk.color_parse(ph[1])
		style = self.l_phase2.get_style().copy()
		style.bg[gtk.STATE_NORMAL] = color
		style.bg[gtk.STATE_ACTIVE] = color
		style.bg[gtk.STATE_PRELIGHT] = color
		self.l_phase2.set_style(style)
		self.l_phase2.show()
		table.attach(self.l_phase2, 1,2,0,1)	
		
		# Select button
		self.cb_select = self.gtk.CheckButton('Select')	###### init properly
		self.cb_select.connect("clicked", self.change_select, {})
		self.cb_select.show()
		table.attach(self.cb_select, 0, 1, 1, 2)
		
		# Buttons	
		#####b_find_phases = self._bouton(self.gtk, "Find phases", self._find_phases, {})
		b_find_phases = self._bouton(self.gtk, "Find phases", self._test, {})
		b_find_phases.show()
		table.attach(b_find_phases, 1,2,1,2)
		
		b_configure_phases = self._bouton(self.gtk, "Configure...", self._test, {})
		b_configure_phases.show()
		table.attach(b_configure_phases, 2,3,1,2)
		
		# This button should be dynamically registered
		#####b_extract_fluo = self._bouton(self.gtk, "Extract fluorescence", self._extract_fluo, {})
		b_extract_fluo = self._bouton(self.gtk, "Extract fluorescence", self._extract_fluo, {})
		b_extract_fluo.show()
		table.attach(b_extract_fluo, 1,2,2,3)

		b_export = self._bouton(self.gtk, "Export", self._export, {})
		b_export.show()
		table.attach(b_export, 2,3,2,3)	
		self.table = table
		return frame
		
	def update_panel(self) :
		"""Function that updates the :guilabel:`Cell Panel` with the last data coming from
		the OpenCv player.
		
		.. note::
			For the moment, only the :guilabel:`phase` label is updated based on the :func:`_get_phase` function"""
		# Update phase button
		#####ph = self._get_phase()		# TMP excluded
		ph = ('temporary message', 'pink')
		self.l_phase2.set_label(ph[0])
		color = self.gtk.gdk.color_parse(ph[1])
		style = self.l_phase2.get_style().copy()
		style.bg[self.gtk.STATE_NORMAL] = color
		style.bg[self.gtk.STATE_ACTIVE] = color
		style.bg[self.gtk.STATE_PRELIGHT] = color
		self.l_phase2.set_style(style)
		self.l_phase2.show()
		
		# Update select checkbox
		#self.auto_toggle = False
		#print (self.is_selected(), self.cb_select.get_active(), self.auto_toggle)
		if self.parent.is_selected() != self.cb_select.get_active() and not self.auto_toggle :
			#print (self.is_selected(), self.cb_select.get_active())
			self.auto_toggle = True
			self.cb_select.set_active(self.parent.is_selected())
			
	def _export(self, source=None, event=None) :
		"""Exports the cell as a sequence of images"""
		gtk = self.gtk
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		response = chooser.run()
		r = False
		if response == gtk.RESPONSE_OK:
			n = self.parent.movie.get_name()
			t = time.strftime("%d-%m-%Y--%Hh%M", time.localtime())
			exp = chooser.get_filename()+'/'+str(n)+'-' + str(self.parent.get_label()) + '-' + t + '/'
			os.mkdir(exp)
			r = True
		elif response == gtk.RESPONSE_CANCEL:
			pass
		chooser.destroy()
		print 'File chose'
		if r :
			self.export_cell(exp)	# Call export function (export the whole movie, gives it a folder URL

	def export_cell(self, folder, warning=True, index=None, obj=None) :
		"""This function saves the cell in a specified folder. It performs the extraction of 
		the cell from the image, and its rotation to save it in a specified format.
		
		:param folder: The path to the folder where the images will be saved. This folder
			must exist and be empty.
		:type folder: str
		
		.. warning::
			This function contains a lot of warnings.
		
		.. warning::
			The code of this function is stil under development, it may not work properly.
		
		.. warning::
			The givent 'folder' must exist, else errors will be raised. Furthermore, this
			folder should be empty since the saving of the images will replace any file that
			has the same name as a saved image. *Be careful*.
		
		.. warning::
			This function locks the control panel since it requires to use the playback.
			
		.. warning::
				The :class:`ct_Cells.Cell` instancemust contain a reference to an instance 
				of the :class:`cv` module.
		
		"""
		gtk = self.gtk
		#parent = self.movie.grid.window_process	# defined if draw_process_grid has been called
		##self.movie.grid.hide_grid()			# Hide grid
		#self.parent.movie.display_m.b_set_grid.set_sensitive(False)	# Blocking grid editing
		
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.connect("destroy", self._destroy_progress)
		
		table = gtk.Table(3,2, True)
		table.show()
		self.dialog.add(table)
		
		if warning :
			self.dialog.set_title("Please check playback options")
			label = gtk.Label("The current playback settings (speed, view mode)\n\
will be used to render the cell. Please correct them before clicking ok.\n\
Please note that for the moment the export runs from the beginning to the\n\
end of the video.")
			table.attach(label, 0, 2,0,1)	
			b_ok = self._bouton(gtk, 'Ok', self._run_export, {'folder': folder, 'index':index, 'obj': obj})		# Too many parameters , useless?
			table.attach(b_ok, 0,1,2,3)
			b_cancel = self._bouton(gtk, 'Cancel', self._destroy_progress, {})
			table.attach(b_cancel, 1,2,2,3)			
			
		else :
			self.dialog.set_title("Exporting cell")
			self._run_export('',folder, index, obj)
		
		# Create the ProgressBar
		self.pbar = gtk.ProgressBar()
		table.attach(self.pbar, 0,2,1,2)
		self.pbar.show()
		self.pbar.set_text('Press Ok when ready')
		
		self.dialog.show_all()
		self.frac = 0
		
	def _run_export(self, w, data) :
		folder = data['folder']
		index = data['index']
		obj = data['obj']
		self.frac = 0
		n = self.parent.movie.get_nb_frames()
		self.running_export = False
		self.first_run = True
		self.export_break = False
		
		cv = self.parent.cv
		
		txt = 'Save file : %s\nCell index : %s' % (str(self.parent.movie.import_export.file), str(self.parent.get_label()))
		f = open(folder + 'data.txt', 'w')
		f.write(txt)
		f.close()
		
		i=0
		#self.timer = gobject.timeout_add(100, self._export_timer, folder, index, obj)
		print "- Computing size"
		l_size = []
		for i in range(n) :
			if i % 100 == 0 :
				print "image %s" % str(i)
			p=self.parent.get_reference_poly(i)
			p=numpy.array(p)
			size = p.max(axis=0)-p.min(axis=0)
			l_size.append(size)
		l_size = numpy.array(l_size)
		rect = l_size.max(axis=0)
		print '- Saving'
		for i in range(n) :
			path = folder + 'image'+str(i) + '.png'
			path_cm = folder + 'image_cm'+str(i) + '.png'
			path_rm = folder + 'image_rm'+str(i) + '.png'
			self.parent.export_frame(i, (path, path_cm, path_rm), rect=rect)			
			
			self.pbar.set_text('image'+str(i) + '.png')
			if i%20 == 0 :
				print 'image'+str(i) + '.png'
			self.parent.movie.set_frame_index(i)
			self.frac = float(i+1)/n
			self.pbar.set_fraction(self.frac)
			if self.export_break :
				break
		self.parent.movie.set_frame_index(0)
		self._destroy_progress('')
		
		
		
	def _export_timer(self, folder, index=None, obj=None) :
		print 'timer'
		"""if self.first_run and obj.index==index :
			self.running_export = True
			self.frac = 0
			self.movie.set_speed(1)				# Speed 1
			self.movie.set_mode('manual')		# Mode manual
			self.movie.set_frame_index(0)		# Set frame index
			self.movie.grid.movie_gtk.lock()	# Lock
			self.movie.grid.hide_grid()	# Lock
			self.first_run = False
			self.progress_data = {'i' : 0}
			self.pbar.set_text('')

		cv = self.cv
		ret = True
		if self.running_export :
			i = self.progress_data['i']
			if i < self.n and obj.index==index :
				f = self.movie.get_frame_displayed()	# The frame currently displayed by opencv
				if f == i :
					#print (i, obj.index, index)
					path = folder + 'image'+str(i) + '.png'
														
					c = self.get_cell()					# Getting OpenCV image
					cv.SaveImage(path, c)
					self.movie.manual_next_frame = True
					self.movie.next_frame()
					self.pbar.set_text('image'+str(i) + '.png')
					if i%20 == 0 :
						print 'image'+str(i) + '.png'
					self.frac = float(i+1)/self.n
					self.pbar.set_fraction(self.frac)
					i+=1
				elif f>i :
					self.movie.set_frame_index(i)
			else :
				ret = False
				self.pbar.set_fraction(0)
				self.movie.grid.movie_gtk.unlock()	# Unlock
				self._destroy_progress()
				gobject.source_remove(self.timer)
				obj.index += 1
			self.progress_data['i'] = i"""
		self.frac = float(self.progress_data['i']+1)/self.n
		self.pbar.set_fraction(self.frac)	
		return True

	def _extract_fluo(self, w, data=None) :
		self.parent.fluorescence.extract_fluo_GTK(self.parent.cv, self.parent.movie.display_m, True)

		
	def _test(self, source=None, event=None) :
		print 'Test... well !'	
	
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	# On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()					# This packs the button into the window (a GTK container).
		return bouton
		
	def change_select(self, widget=None, data=None, state=None) :
		"""Function that switches between two states of the cell : selected and not selected
		The selected flag will be used when performing batch processing of the cell.
		
		.. note::
			This function is called when the select button is clicked, and the 'widget' and
			'data' parameters are *not* required to run the function.
			
		.. note::
			For the moment, the selected flag has no effect."""
		print 'toggle!'
		if not self.auto_toggle :	
			if state != None :
				self.parent.set_selected(state)
				#self.selected = state
			else :
				self.parent.set_selected(not self.parent.is_selected())
				#self.selected = not self.selected
		self.auto_toggle = False
		
	def _destroy_progress(self, w, data=None) :
		#self.parent.movie.grid.movie_gtk.b_set_grid.set_sensitive(True)
		self.export_break = True
		self.dialog.destroy()

