# General object managing object evolving through time
# This class should replace most of the function of ct_Cells
import copy

class TimeTrack :
	def __init__(self, start, stop) :
		self.start = start
		self.stop = stop
		
		self.key_objects = []
		self.key_frames = []
		
	def get_save_dict(self) :
		dic = {}
		dic['start'] = {'desc' : 'Beginning time of the polygon', 'value' : self.start}
		dic['stop'] = {'desc' : 'Ending time of the polygon', 'value' : self.stop}
		dic['key_objects'] = {'desc' : 'List of key objects (e.g: polygons)', 'value' : self.key_objects}
		dic['key_frames'] = {'desc' : 'List of key frames', 'value' : self.key_frames}
		return dic

	def set_save_dict(self, dic) :
		self.start = dic['start']['value']
		self.stop = dic['stop']['value']
		self.key_objects = dic['key_objects']['value']
		self.key_frames = dic['key_frames']['value']

	def add_object(self, obj, frame) :
		# Adjust opening/closing time
		if frame < self.start :
			self.start = frame
		elif frame > self.stop :
			self.stop = frame

		# Get the index of the key_object or create it
		if frame in self.key_frames :
			#print '-- getting frame index'
			i = self.key_frames.index(frame)
		elif self.key_frames==[] :
			#print '-- creating first frame'
			self.key_frames.append(frame)
			self.key_objects.append([])
			#self.rect.append([])
			i=0	
		else :
			#print '-- creating new frame'
			i=self.create_frame(frame)
		
		# Adding the polygon at the right place
		self.key_objects[i] = obj			

	def create_frame(self, im) :
		"""Function that inserts a frame by copying the previous one
		That function doesn't check if the frame already exists
		it MUST have been checked before !"""
		i=-1
		if im > self.key_frames[-1] :
			self.key_frames.append(im)
			self.key_objects.append(copy.copy(self.key_objects[-1]))	# /!\ Don't insert blank frame, COPY the previ
			#self.rect.append(copy.copy(self.rect[-1]))
			i = len(self.key_frames)-1
			#print "frame " + str(im) +" inserted at position " + str(len(self.key_frames)-1)	# Debug
		elif im < self.key_frames[0] :
			self.key_frames.insert(0,im)
			self.key_objects.insert(0, copy.copy(self.key_objects[0]))	# /!\ Don't insert blank frame, COPY the previ
			#self.rect.insert(0, copy.copy(self.rect[-1]))
			#print "frame " + str(im) +" inserted at position " + str(len(self.key_frames)-1)	# Debug
			i=0
		else :
			i=1
			while i <= len(self.key_frames) :
				if self.key_frames==[] or self.key_frames[i] > im :	# Surrounding frames : [i-1, i]
					obj = self.interpolate(i, im)		# Interpolate poly
					self.key_frames.insert(i,im)		# Register frames
					self.key_objects.insert(i, copy.copy(obj))
					#self.rect.insert(i, []) 		# Save poly
					#self.get_rect(im, True)			# Regenerate rect
					#print "frame " + str(im) +" inserted at position " + str(i)	# Debug
					break
				i +=1
		if len(self.key_frames) != len(self.key_objects) :
			print "INCOHERENT SIZES"
		return i
		
	def get_index(self, im) :
		"""Returns the index in the list for the right image, or False
		if the index doesn't exists or if it is out of the lifespan of the object"""
		i = None
		if im < self.start or im>self.stop :
			i=-1
		elif im in self.key_frames :
			i=self.key_frames.index(im)
		return i

	def get_object(self, im) :
		"""Function that returns the object corresponding to the image"""
		i=self.get_index(im)
		if self.key_frames == [] :
			print 'no key frames'
			pass
		elif im < self.key_frames[0] :
			p = self.key_objects[0]
			#print "TOO EARLY"	# Debug
		elif im > self.key_frames[-1] :
			p = self.key_objects[-1]
			#print "TOO LATE"	# Debug
		elif i==None :
			j=0
			while j<len(self.key_frames) and self.key_frames[j] < im :
				j+=1	# Gives the superior bound
			p = self.interpolate(j, im)
		else :
			p = self.key_objects[i]
		return p
		
	def interpolate(self, next_idx, t) :
		"""Function that returns the linearily interpolated polygon
		This function should be overwriten to handle non-polygon objects, 
		or to process a non-linear interpolation"""
		t1 = self.key_frames[next_idx-1]	# Calculate time delta
		t2 = self.key_frames[next_idx]
		x = float(t-t1)/(t2-t1)		# Weighted average
		p = []
		for pp in range(len(self.key_objects[next_idx])) :
			p1 = self.key_objects[next_idx-1][pp]	# Extract points
			p2 = self.key_objects[next_idx][pp]
			pt = [-1,-1]
			pt[0] = int( p1[0] + (p2[0]-p1[0])*x )	# Linear interpolation
			pt[1] = int( p1[1] + (p2[1]-p1[1])*x )
			p.append(pt)
		return p
