# Module managing save/open features
# This module is an interface between the CellTracking modules and, for instance, 
#+ a HDF5 file.
# If everything work well, this module should be usable as a basement to build a scripting
#+ language giving access to all the features of CellTracking

class ImportExport :
	def __init__(self, movie) :
		self.movie = movie
		self.file = None 	# The save file
		self.accepted_formats = ['ct', 'txt']

	def prepare_export(self) :
		"""This function extracts all the dictionnaries in order to format them
		before exporting them"""
		print "-- Preparing export"
		print "--- Channels data"
		dic_ch = self.movie.channels.get_save_dict()
		print "--- Grid data"
		dic_gr = self.movie.grid.get_save_dict()
		print "--- Cells data"
		dic_ce = self.movie.cells.get_save_dict()
		print "--- Packing all together"
		dic = {}
		dic['channels'] = {'desc' : 'Channels data', 'value' : dic_ch}
		dic['grid'] = {'desc' : 'Grid data', 'value' : dic_gr}
		dic['cells'] = {'desc' : 'Cells data', 'value' : dic_ce}
		#dic['phases'] = {'desc' : 'Phases data', 'value' : dic_ph}
		print "-- Done"
		return dic
	def load_all(self) :
		"""Loads everything from the selected file
		Except the channels that have a different loading procedure"""
		if self.file != None :
			dic = self.main_dict
			self.movie.grid.load_save_dict(dic['grid']['value'])		# Load grid
			self.movie.cells.set_save_dict(dic['cells']['value'])
			
	def load_file(self, filename) :
		#self.file_name = filename.split('/')[-1].split('.')[0]
		ext = filename.split('.')[-1]
		r = False
		if ext in self.accepted_formats :
			r = True
			self.file =  filename
			import pickle
			f = open(filename, 'rb')
			self.main_dict = pickle.load(f)
		return r
	def get_dict(self, key) :
		"""Returns an unpacked dictionary (first level only) :
		supported keys = first level keys : 'channels', 'grid', 'cells'"""
		r = None
		if self.file != None :
			r = self.main_dict[key]['value']
		else :
			print """/!\ WARNING, no input file selected"""
		return r
		
	def add_gtk_instance(self, gtk) :
		self.gtk = gtk
	def add_save_buttons(self) :
		l = self._bouton(self.gtk, 'Save', self._b_save, {})
		ll = self._bouton(self.gtk, 'Save as', self._b_save_as, {})
		self.movie.display_m.add_button(l)
		self.movie.display_m.add_button(ll)
	def _b_save_as(self, w, data=None) :
		gtk = self.gtk
		dialog = gtk.FileChooserDialog("Save a CellTracking file", None, gtk.FILE_CHOOSER_ACTION_SAVE, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK)) 
		filter_grid = gtk.FileFilter()
		filter_grid.set_name("CellTracking file (python pickle file) - *.ct")
		filter_grid.add_pattern("*.ct")
		dialog.add_filter(filter_grid)
		filter_txt = gtk.FileFilter()
		filter_txt.set_name("Text file (python pickle file) - *.txt")
		filter_txt.add_pattern("*.txt")
		filter_h5 = gtk.FileFilter()
		filter_h5.set_name("HDF file - *.h5")
		filter_h5.add_pattern("*.h5")
		dialog.add_filter(filter_h5)
		filter_any = gtk.FileFilter()
		filter_any.set_name("any file")
		filter_any.add_pattern("*")
		dialog.add_filter(filter_any)
		
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			filename = dialog.get_filename()
			ext = filename.split('.')[-1]
			if ext != 'txt' and ext != 'ct' and ext != 'h5' :
				filename += '.ct'
			self.file = filename
			self._save()
			# SAVE - Warning on replace
		dialog.destroy()

	def _b_save(self, w, data=None) :
		if self.file == None :
			self._b_save_as('','')
		else :
			self._save()
		
	def _save(self) :
		dic = self.prepare_export()
		import pickle
		f = open(self.file, 'wb')		# Create pickle object
		pickle.dump(dic, f, -1)
		f.close()
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	 # On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()# This packs the button into the window (a GTK container).
		return bouton
