#!/usr/bin/python
# -*- coding: utf-8 -*-
# this modules contains the required function to extract cellular cycle from the movie
# MW 2012 GPLv3+

import gobject, time, imp
import numpy as np
class Phases :
	"""This class is instanciated by a cell object,
	it contains the functions required to determine if the cell exist, 
	and in which part of the cell cycle it lives"""
	
	def __init__(self, cell, movie) :
		self.find_phases_order = ['is_cel', 'nebd']		# The order in which the phases will be searched
		self.phases_list = []
		self.phases_dict = {}
		self.phases_list_p = [("./modules/modules_phases/ph_is_cell.py", 'ph_is_cell'),
								("./modules/modules_phases/ph_NEBD.py", 'ph_NEBD'),
								('./modules/modules_phases/ph_anaphase.py', 'ph_anaphase')]		# A list of feature modules that are being instanciated

		for i in self.phases_list_p :
			ph_m = imp.load_source(i[1], i[0])			# Load
			ph = ph_m.Phase(cell, movie)
			self.phases_list.append((ph.get_s_name(), ph))	# Register in list i[0] is CLEARLY not the right index -> use get_name()
			self.phases_dict[ph.get_s_name()] = ph			# Register in dict		
		
		self.cell = cell
		self.movie = movie
		self.n = self.movie.get_nb_frames()

	
	def get_phases_list(self) :
		return self.phases_list
	def get_phase(self) :
		"""THIS FUNCTION MUST DISAPPEAR SINCE THE PHASES MODULE HAS BEEN REORGANIZED
		FOR THE MOMENT IT RETURNS A BINDING TO THE IS_CELL MODULE"""
		i = self.movie.get_frame_displayed()
		ph = self.phases_dict['is_cel']
		ret = (ph.get_phase(i)[2], ph.get_phase(i)[3])
		return ret
		
	def get_save_dict(self) :
		ph_l = self.get_phases_list()
		l = []
		for ph in ph_l :
			dic = {}
			name = ph[0]
			save_dic = ph[1].get_save_dict()
			dic['name'] = {'desc' : 'short name of the phase', 'value' : name}
			dic['phase_data'] = {'desc' : 'The internal parameters of the concerned phase module', 'value' : save_dic}
			l.append(dic)
		out = {}
		out['phases'] = {'desc' : "A list of all the initiated phases, with their prameters", 'value' : l}
		return out
	def set_save_dict(self, dic) :
		ph_l = dic['phases']['value']
		for ph in ph_l :
			self.phases_dict[ph['name']['value']].set_save_dict(ph['phase_data']['value'])
	
	def find_phases_GTK(self, cv, movie_gtk, warning=True) :
		"""Functions that locks everything before running find_phases"""
		gtk = movie_gtk.gtk
		self.movie_gtk = movie_gtk
		self.movie.grid.hide_grid()			# Hide grid
		movie_gtk.b_set_grid.set_sensitive(False)	# Blocking grid editing
		
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.connect("destroy", self._destroy_progress)

		table = gtk.Table(3,2, True)
		table.show()
		self.dialog.add(table)
		
		if warning :
			self.dialog.set_title("Please check playback options")		
			label = gtk.Label("The current playback settings (speed, view mode)\n\
will be used to render the cell. Please correct them before clicking ok.\n\
Please note that for the moment the export runs from the beginning to the\n\
end of the video.")
			table.attach(label, 0, 2,0,1)
			b_ok = self._bouton(gtk, 'Ok', self._b_ok, cv)
			table.attach(b_ok, 0,1,2,3)
			b_cancel = self._bouton(gtk, 'Cancel', self._destroy_progress, {})
			table.attach(b_cancel, 1,2,2,3)			
		else :
			self.dialog.set_title("Exporting cell")
			self._b_ok('',cv)
			
		# Create the ProgressBar
		self.pbar = gtk.ProgressBar()
		table.attach(self.pbar, 0,2,1,2)
		self.pbar.show()
		self.pbar.set_text('Press Ok when ready')

		self.first_run_progress = True
		self.dialog.show_all()
		self.frac = 0
		
	def _b_ok(self, w, cv=None) :
		#self.movie.set_locked(True)
		self.timer = gobject.timeout_add(100, self.find_phases,cv)
		
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# New button
		bouton.connect("clicked", function, dico)	# Connnection
		bouton.show()					# This packs the button into the window (a GTK container).
		return bouton 
		
	def _destroy_progress(self, w, data=None) :
		self.movie_gtk.b_set_grid.set_sensitive(True)
		self.dialog.destroy()

		
		
	def find_phases(self, cv) :
		"""Function that find phases for each image
		it can be called by a timer, which explains the particular structure"""
		if self.first_run_progress :			# Ran once
			self.first_run_progress = False
			self.run_progress = True
			k = len(self.find_phases_order)
			self.progress_data = {'i' : 0,
								'nb_modules' : k,
								'mod_i' : 0,
								'increment' : 1.0/self.n/k
								}
			self.frac = 0
			self.movie.grid.movie_gtk.lock()	# Lock
			self.movie.grid.hide_grid()	# Lock
			
		if self.run_progress :					# ran once per module finding phases
			self.movie.set_speed(1)				# Speed 1
			self.movie.set_mode('manual')		# Mode manual
			self.movie.set_frame_index(0)		# Set frame index
			self.run_progress = False
			finding_module_n = self.find_phases_order[self.progress_data['mod_i']]
			finding_module = self.phases_dict[finding_module_n]
			self.pbar.set_text('Running module : ' + finding_module.get_name())
			self.progress_data['i'] = 0
		
		ret = True
		i = self.progress_data['i']
		
		if i < self.n :
			f = self.movie.get_frame_displayed()	# The frame currently displayed by opencv
			if f == i  :
				mod_name = self.find_phases_order[self.progress_data['mod_i']]
				mod = self.phases_dict[mod_name]				# Get the module
				mod.find_phase(i)# Call the function
				
				# Playback
				self.movie.manual_next_frame = True
				self.movie.next_frame()
				if i%20 == 0 :
					print 'image '+str(i)
				incr = self.progress_data['increment']
				#self.frac = float(i+1)*incr
				frac = self.pbar.get_fraction()
				self.pbar.set_fraction(frac+incr)
				i+=1
				
			elif f>i :
				self.movie.set_frame_index(i)
		else :
			if self.progress_data['mod_i'] +1 >= self.progress_data['nb_modules'] :	# Ending the research
				ret = False
				self.pbar.set_fraction(0)
				self.movie.grid.movie_gtk.unlock()	# Unlock
				self._destroy_progress('', '')
				gobject.source_remove(self.timer)
			else :		# Initiating next module
				self.run_progress = True
				self.progress_data['mod_i'] += 1
		self.progress_data['i'] = i
		return ret
