# This is a simple brightness/contrast manager for CellTracking
# MW Aug 2012 - GPLv3+
import cv
from numpy import *

class Contrast :
	def __init__(self, movie) :
		self.movie = movie
		self.panel_visible = False
		self.contrast_active = False
		self.fast_contrast = False		# Choses the contrast algorithm
		self.index_contrast = 1
	def add_gtk_instance(self, gtk) :
		self.gtk = gtk
	def add_contrast_button(self) :
		l = self._bouton(self.gtk, 'Brightness/Contrast', self._show_contrast_panel, {})
		self.movie.display_m.add_button(l)
	def _show_contrast_panel(self, w, data=None) :
		if not self.panel_visible :
			self.window = self.gtk.Window(self.gtk.WINDOW_TOPLEVEL)
			self.window.set_size_request(110, 20)
			self.window.set_title('Adjust brightness/contrast')
			self.window.show()
			box = self.gtk.HBox()
			box.show()
			self.window.add(box)
			b_plus = self._bouton(self.gtk, '-', self._change_contrast, -1)
			box.pack_start(b_plus, False, False)
			b_plus = self._bouton(self.gtk, '+', self._change_contrast, +1)
			box.pack_start(b_plus, False, False)
			b_auto = self._bouton(self.gtk, 'Auto', self._auto_contrast, {})
			box.pack_start(b_auto, False, False)
			b_reset = self._bouton(self.gtk, 'Reset', self._reset_contrast, {})
			box.pack_start(b_reset, False, False)
			self.panel_visible = True
		else :
			self.window.destroy()	# Destroy
			self.panel_visible = False
	def overlay(self, image) :
		if self.contrast_active :
			if self.index_contrast != 1 :
				cv.ConvertScale(image, image, self.index_contrast)
			elif not self.fast_contrast :
				#print 'autocontrast'
				m = cv.GetMat(image)
				m,cdf=self.histeq(asarray(m))
				im = cv.GetImage(cv.fromarray(m.astype('uint8')))
				cv.Convert(im, image)

				#print 'done'
			else :
				pass
		return image
	def _change_contrast(self, w, increment) :
		self.contrast_active = True
		self.index_contrast += increment
	def _auto_contrast(self, w, data=None) :
		self.index_contrast = 1
		self.contrast_active = True
	def _reset_contrast(self, w, data=None) :
		self.contrast_active = False
		self.index_contrast = 1
	
	def histeq(self,im,nbr_bins=256):
	   # From http://www.janeriksolem.net/2009/06/histogram-equalization-with-python-and.html
	   #get image histogram
	   imhist,bins = histogram(im.flatten(),nbr_bins,normed=True)
	   cdf = imhist.cumsum() #cumulative distribution function
	   cdf = 255 * cdf / cdf[-1] #normalize

	   #use linear interpolation of cdf to find new pixel values
	   im2 = interp(im.flatten(),bins[:-1],cdf)
	   return im2.reshape(im.shape).astype('uint8'), cdf
		
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	 # On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()# This packs the button into the window (a GTK container).
		return bouton 
